import { Body, Controller, Delete, Get, Param, Patch, Post, Query, Req, Request, UploadedFile, UploadedFiles, UseGuards, UseInterceptors } from '@nestjs/common';
import { FileFieldsInterceptor, FileInterceptor } from '@nestjs/platform-express';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';
import { getMulterConfig } from 'src/core/multer.config';
import { UploadedMulterFileI } from 'src/modules/SpacesModule/SpacesService';
import { Constants } from 'src/utils/constants';
import { CreateConfirmImportProductDto } from './dto/create-confirm-import-product.dto';
import { CreateConfirmStockInProductDto } from './dto/create-confirm-stockin-product.dto';
import { CreateImportProductDto } from './dto/create-import-product.dto';
import { CreateProductDto } from './dto/create-product.dto';
import { UpdateProductDto } from './dto/update-product.dto';
import { ProductService } from './product.service';

@Controller({
    path: 'product',
    version: Constants.API_VERSION_1
})
export class ProductController {

    constructor(
        private readonly productsService: ProductService,
    ) { }


    @Post()
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'productImages', maxCount: 4 },
            { name: 'featuredImage', maxCount: 1 }
        ],
    ))
    create(
        @Body() dto: CreateProductDto,
        @UploadedFiles() files: {
            productImages?: UploadedMulterFileI,
            featuredImage?: UploadedMulterFileI,
        }) {
        return this.productsService.create(dto, files);
    }

    @Post('/new-stock-in')
    @UseInterceptors(
        FileInterceptor(
            'excelFile',
            getMulterConfig(
                './uploads/product-stockin'
            )
        )
    )
    importStockInProduct(@Body() importProductDto: CreateImportProductDto, @UploadedFile() file: Express.Multer.File) {
        return this.productsService.importProductStockService(importProductDto, file);
    }

    @Post('/confirm-stockin-product')
    confirmStockinProduct(@Body() dto: CreateConfirmStockInProductDto) {
        return this.productsService.confirmStockInProductService(dto);
    }

    @Post('/import-product')
    @UseInterceptors(
        FileInterceptor(
            'excelFile',
            getMulterConfig(
                './uploads/product-dump'
            )
        )
    )
    importProduct(@Body() importProductDto: CreateImportProductDto, @UploadedFile() file: Express.Multer.File) {
        // console.log('createProductDto = ', importProductDto);
        return this.productsService.importProductService(importProductDto, file);
    }


    @Post('/confirm-import-product')
    confirmImportProduct(@Body() importProductDto: CreateConfirmImportProductDto) {
        console.log('createProductDto = ', importProductDto);
        return this.productsService.confirmImportProductService(importProductDto);
    }

    @Get('/export')
    async exportProducts() {
        return await this.productsService.exportProductService();
    }

    // Need filtering
    @Get('/get-all')
    async filterByProductName(
        @Query('limit') limit: any,
        @Query('page') page: any,
        @Query('keyword') keyword?: any,
        @Query('routeName') routeName?: any,
        @Query('parentCategoryId') parentCategoryId?: any,
        @Query('subCategoryId') subCategoryId?: any,
        @Query('childCategoryId') childCategoryId?: any,
        @Query('maxPrice') maxPrice?: any,
        @Query('minPrice') minPrice?: any,
        @Query('color') color?: any,
        @Query('size') size?: any,
        @Query('tag') tag?: any,
        @Query('material') material?: any,
        @Query('highlight') highlight?: any,
        @Query('sortBy') sortBy?: any,
    ) {
        return await this.productsService.filterByProductName(
            limit,
            page,
            keyword,
            routeName,
            parentCategoryId,
            subCategoryId,
            childCategoryId,
            maxPrice,
            minPrice,
            color,
            size,
            tag,
            material,
            highlight,
            sortBy,
        );
    }

    @Get()
    async findAll(
        @Query('limit') limit: number,
        @Query('page') page: number
    ) {
        return await this.productsService.findAll(limit, page);
    }

    @Get('/:id') // for public viewer
    async findOne(@Param() id: string) {
        return await this.productsService.findOne(id);
    }

    @Patch('/:id')
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: 'productImages', maxCount: 4 },
            { name: 'featuredImage', maxCount: 1 }
        ],
    ))
    async updateProduct(
        @Param('id') id: string,
        @Body() updateProductDto: UpdateProductDto,
        @UploadedFiles() files: {
            productImages?: UploadedMulterFileI,
            featuredImage?: UploadedMulterFileI,
        }
    ) {
        return await this.productsService.updateOneProduct(id, updateProductDto, files);
    }

    @Delete(':id')
    async remove(@Param('id') id: string) {
        return await this.productsService.remove(id);
    }
}
